;----------------------------------------------------------------------------
; EBook         STF
; 
;----------------------------------------------------------------------------
; See_also      HestonCall, HestonVanilla, HestonCallMC 
;
;----------------------------------------------------------------------------
; Macro         STFfft02
; 
;----------------------------------------------------------------------------
; Description   a) comparison of prices obtained by Monte Carlo method, analytical formula 
;								and FFT method in Heston model.
;               b) percentage difference between the prices obtained with
;               the FFT method and analytical fomula for Heston model. 
;
;---------------------------------------------------------------------------- 
proc()=makeplot()
  S=100
  K=120:80
  T=1
  r=0.02
  kappa = 10.0
  theta = 0.2
  sigma = 0.7
  rho = -0.5
  v0 = 0.2

  I=500
  M=5000

  y=HestonCall(S,K,T,r,kappa,theta,sigma,rho,(v0))
  disp1=createdisplay(1,1)
  p=setmask(K~y,"line","red")
  show(disp1,1,1,p)

  K=(12:8)*10
  i=1
  while(i<=rows(K))
    y1=HestonVanilla(1,S,K[i],v0,sigma,r,0,T,kappa,theta,0,rho)
    p1=setmask(K[i]~y1,"large","cross")
    adddata(disp1,1,1,p1)
    i=i+1
  endo
  setgopt(disp1,1,1,"border", 0,"xlabel","strike price","ylabel","option price" )
 
  i=1
  while(i<=rows(K))
    y=HestonCallMC(S,K[i],T,r,kappa,theta,sigma,rho,v0,I,M)
    p=setmask(K[i]~y,"fillcircle","blue")
    adddata(disp1,1,1,p)
    i=i+1
  endo
  setgopt(disp1,1,1,"border", 0,"xlabel","strike price","ylabel","option price" )
  setgopt(disp1,1,1, "title", "Option prices in the Heston model")
  
  K=80+(0:80)/2

  T1=matrix(size(K))
  T2=matrix(size(K))

  i=1
  while(i<=rows(K))

    lambda = 0
    t1 = HestonVanilla(1,S,K[i],v0,sigma,r,0,T,kappa,theta,lambda,rho)
    t2 = HestonCall(S,K[i],T,r,kappa,theta,sigma,rho,v0)

    T1[i]=t1
    T2[i]=t2
    i=i+1
  endo

  p1=setmask(K~(100*(T1-T2)./T2),"line","red")
  disp2=createdisplay(1,1)
  show(disp2,1,1,p1)
  setgopt(disp2,1,1, "title", "(Analytical - FFT)/Analytical [%]","xlabel","strike price","ylabel","MAPE")
  setgopt(disp2,1,1, "xvalue", 0|1, "yvalue", 0|1,"ymajor",0.1)
  setgopt(disp2,1,1, "border", 0)

endp

library("finance")
randomize(13)
makeplot()

