;----------------------------------------------------------------------------
; EBook         STF
; 
;----------------------------------------------------------------------------
; See_also      BatesCall, BatesPut 
;
;----------------------------------------------------------------------------
; Macro         STFfft03
; 
;----------------------------------------------------------------------------
; Description   plot of the fitted Bates model to the implied volatilities 
;               surface of DAX optoins on January, 4th 1999. 
;---------------------------------------------------------------------------- 

proc()=plotresults(data,x)

  a=discrete(data[,4:3|1])
  T = a.xr[,1]
  r = a.xr[,2]
  S = a.xr[,3] 
  
  a =discrete(data[,2] )
  K =a .xr

  price  =-1* matrix(rows(K),rows(T))
  option =-1* matrix(rows(K),rows(T))
  
  j=1
  while(j<=rows(data))
    nrow=paf(1:rows(K),K==data[j,2] )
    ncol=paf(1:rows(T),T==data[j,4] )
    option[nrow, ncol]= data[j,6]
    price [nrow, ncol]= data[j,5] 
    j=j+1
  endo

  iv = ImplVola(data) 
  imvol =-1* matrix(rows(K),rows(T))

;create matrix of IV 
  j=1
  while(j<=rows(data))
    nrow=paf(1:rows(K),K==data[j,2] )
    ncol=paf(1:rows(T),T==data[j,4] )
    imvol [nrow, ncol]= iv[j] 
    j=j+1
  endo

  d = createdisplay(2,2)

  i=2
  k = paf(1:rows(K),price[,i]>=0)
  observations = setmask((K)[k]~imvol[k,i],"red")
  tk=((0:25)/25)*(max((K)[k])- min((K)[k]))+min((K)[k])
  opk=BatesCall((S)[i],tk,(T)[i],(r)[i],x[1,],x[2,],x[3,],x[4,],x[5,],x[6,],x[7,],x[8,])
  model = (S)[i]*matrix(rows(tk))
  model = model~tk~(matrix(rows(tk))*((r)[i]~(T)[i]))
  model = model~opk~(matrix(rows(tk)))
  model = ImplVola(model)
  model = setmask(tk~model,"line","blue")
  head ="Time to maturity T="+ string("%1.4f",T[i])

  show(d,1,1,observations,model)
  setyaxis(d,1,1,0,0.7)
  setgopt(d,1,1, "xvalue", 0|1, "yvalue", 0|1,"xmajor",1000,"ymajor",0.1)
  setgopt(d,1,1,"border", 0)
  setgopt(d,1,1, "xlabel", "strike", "ylabel", "implied volatility")
  setheadline(d,1,1,head)

  i=3
  k = paf(1:rows(K),price[,i]>=0)
  observations = setmask((K)[k]~imvol[k,i],"red")
  tk=((0:25)/25)*(max((K)[k])- min((K)[k]))+min((K)[k])
  opk=BatesCall((S)[i],tk,(T)[i],(r)[i],x[1,],x[2,],x[3,],x[4,],x[5,],x[6,],x[7,],x[8,])
  model = (S)[i]*matrix(rows(tk))
  model = model~tk~(matrix(rows(tk))*((r)[i]~(T)[i]))
  model = model~opk~(matrix(rows(tk)))
  model = ImplVola(model)
  model = setmask(tk~model,"line","blue")
  head ="Time to maturity T="+ string("%1.4f",T[i])

  show(d,1,2,observations,model)
  setyaxis(d,1,2,0,0.7)
  setgopt(d,1,2, "xvalue", 0|1, "yvalue", 0|1,"xmajor",1000,"ymajor",0.1)
  setgopt(d,1,2,"border", 0)
  setgopt(d,1,2, "xlabel", "strike", "ylabel", "implied volatility")
  setheadline(d,1,2,head)

  i=4
  k = paf(1:rows(K),price[,i]>=0)
  observations = setmask((K)[k]~imvol[k,i],"red")
  tk=((0:25)/25)*(max((K)[k])- min((K)[k]))+min((K)[k])
  opk=BatesCall((S)[i],tk,(T)[i],(r)[i],x[1,],x[2,],x[3,],x[4,],x[5,],x[6,],x[7,],x[8,])
  model = (S)[i]*matrix(rows(tk))
  model = model~tk~(matrix(rows(tk))*((r)[i]~(T)[i]))
  model = model~opk~(matrix(rows(tk)))
  model = ImplVola(model)
  model = setmask(tk~model,"line","blue")
  head ="Time to maturity T="+ string("%1.4f",T[i])

  show(d,2,1,observations,model)
  setyaxis(d,2,1,0,0.7)
  setgopt(d,2,1, "xvalue", 0|1, "yvalue", 0|1,"xmajor",1000,"ymajor",0.1)
  setgopt(d,2,1, "xlabel", "strike", "ylabel", "implied volatility")
  setgopt(d,2,1,"border", 0)
  setheadline(d,2,1,head)

  i=5
  k = paf(1:rows(K),price[,i]>=0)
  observations = setmask((K)[k]~imvol[k,i],"red")
  tk=((0:25)/25)*(max((K)[k])- min((K)[k]))+min((K)[k])
  opk=BatesCall((S)[i],tk,(T)[i],(r)[i],x[1,],x[2,],x[3,],x[4,],x[5,],x[6,],x[7,],x[8,])
  model = (S)[i]*matrix(rows(tk))
  model = model~tk~(matrix(rows(tk))*((r)[i]~(T)[i]))
  model = model~opk~(matrix(rows(tk)))
  model = ImplVola(model)
  model = setmask(tk~model,"line","blue")
  head ="Time to maturity T="+ string("%1.4f",T[i])

  show(d,2,2,observations,model)
  setyaxis(d,2,2,0,0.7)
  setgopt(d,2,2, "xvalue", 0|1, "yvalue", 0|1,"xmajor",1000,"ymajor",0.1)
  setgopt(d,2,2, "xlabel", "strike", "ylabel", "implied volatility")
  setgopt(d,2,2,"border", 0)
  setheadline(d,2,2,head)

  fit= 0 
  
  numopt = sum((sum(option>=0))')
; numopt

  i=1
  while(i<=rows(T))
    if ( max(option[,i])>=0)
      k = paf(1:rows(K),option[,i]>=0)
      modelprice=BatesCall((S)[i],(K)[k],(T)[i],(r)[i],x[1,],x[2,],x[3,],x[4,],x[5,],x[6,],x[7,],x[8,])

      modelpriceput = (option[k,i]==0).*(modelprice  + (K)[k].*exp(-(r)[i]*(T)[i])-(S)[i]) 
      modelpricecall= modelprice .*(option[k,i]==1)
      fit = fit + sum(((modelpriceput+ modelpricecall -price[k,i] ).^2) ./(price[k,i]).^2)
    endif
    i=i+1
  endo

  fit = fit/numopt
  fit

endp


library("finance")
data=read("volsurfdata2.dat")
lambda= 0.1326 
delta = 0.00040189  
kbar = -0.034199
kappa =4.2312 
theta = 0.16953
sigma = 1.3941 
rho = -0.55479
v0 =  0.10175 
x = lambda|delta|kbar|kappa|theta|sigma|rho|v0
plotresults(data,x)

